package com.codigoparallevar.minicards.types.connectors.Wiring;

import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;

import com.codigoparallevar.minicards.ScrolledCanvas;
import com.codigoparallevar.minicards.types.Drawable;
import com.codigoparallevar.minicards.types.Dropper;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.connectors.input.InputConnector;
import com.codigoparallevar.minicards.types.wireData.WireDataType;

public class Wire<T extends WireDataType, InputConnectorType extends InputConnector<T, InputConnectorType>>
        implements Moveable, Drawable {

    private final Dropper _dropper;
    private int _xinit;
    private int _yinit;
    private int _xend;
    private int _yend;
    private final static int _pathRunWay = 100;
    protected InputConnectorType _attachedTo = null;

    public Wire(Dropper dropper, int xInit, int yInit) {
        _dropper = dropper;
        _xinit = xInit;
        _yinit = yInit;

        _xend = xInit;
        _yend = yInit;
    }

    public void moveStart(int x, int y){
        _xinit = x;
        _yinit = y;
    }

    @Override
    public void moveEnd(int x, int y) {
        _xend = x;
        _yend = y;
    }

    @Override
    public void drop(int x, int y) {
        _dropper.drop(this);
    }

    @Override
    public void draw(ScrolledCanvas canvas, boolean devMode) {
        final Path samplePath = new Path();

        samplePath.moveTo(_xinit, _yinit);
        samplePath.cubicTo(
                _xinit + _pathRunWay, _yinit,
                _xend - _pathRunWay, _yend,
                _xend, _yend);

        Paint pathPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        pathPaint.setColor(Color.GREEN);
        pathPaint.setStrokeWidth(5.0f);
        pathPaint.setStrokeCap(Paint.Cap.ROUND);
        pathPaint.setStyle(Paint.Style.STROKE);

        canvas.drawPath(samplePath, pathPaint);
    }

    public int getXEnd() {
        return _xend;
    }

    public int getYEnd() {
        return _yend;
    }

    public void attachTo(InputConnectorType resultPoint) {
        _attachedTo = resultPoint;
        _xend = resultPoint.getX();
        _yend = resultPoint.getY();
        resultPoint.getAttachment(this);
    }

    public InputConnector getAttachedTo() {
        return _attachedTo;
    }

    public void unlink() {
        _attachedTo = null;
        _dropper.unlinkWire(this);
    }
}
